@@ -11,7 +11,7 @@
 
 #from yappy.parser import *
 import sys, string
-from  parser import *
+from yappy.parser import *
 ############## Demos  #####################
 
 class SimpleExp(Yappy):
@@ -182,7 +182,7 @@ class SimpleExpAmb(SimpleExp2):
         sinal = "[+-]"
         integer ="\d" 
         tokenize=[("(%s)+"%integer,lambda x: ("id",int(x))),
-               ("%s"%sinal,lambda x: ("add_op",self.make_op(x)),("add_op",100,'left')),
+               (sinal,lambda x: ("add_op",self.make_op(x)),("add_op",100,'left')),
                ("[*/]",lambda x: ("mul_op",self.make_op(x)),("mul_op",200,'left')),
                ("\(|\)",lambda x: (x,x)) ]
 
@@ -228,8 +228,9 @@ class ListAVG(Yappy):
     """A parser  for transforming a list atrib=value into a python dictionary """
     def __init__(self,no_table=0, table='Listavg.tab'):
         grammar =   """
-            E -> ( ) {{self.EmptyDict}} | ( AVL ) {{self.ParSemRule}} ;
-            AVL ->   AV , AVL  {{DefaultSemRule}} | AV {{EmptySemRule}} ;
+            E -> ( ) {{self.EmptyDict}};
+            E ->  ( AVL ) {{self.ParSemRule}} ;
+            AVL ->   AV , AVL  | AV {{EmptySemRule}} ;
             AV -> tok = tok {{ self.AddItem }};
             """
     
@@ -272,7 +273,7 @@ class ListAVG1(ListAVG):
         grammar =   """
             E -> ( ) {{self.EmptyDict}};
             E -> ( AVL ) {{self.ParSemRule}} ;
-            AVL ->   AV , AVL  {{DefaultSemRule}} | AV {{EmptySemRule}} ;
+            AVL ->   AV , AVL   | AV {{EmptySemRule}} ;
             AV -> tok = tok {{ self.AddItem }};
             """
         tokenize = [
@@ -318,7 +319,7 @@ class RegExp(Yappy):
      rules are dummy..."""
         grammar = grules([("r -> r | r",self.OrSemRule),
                         ("r -> r . r",self.ConcatSemRule),
-                    ("r -> r *",self.StarSemRule),
+                    ("r -> r *",self.StarSemRule, (300,'left')),
                     ("r -> ( r )",self.ParSemRule),
                     ("r -> id",self.BaseSemRule),
                    ])
@@ -326,7 +327,7 @@ class RegExp(Yappy):
                     ("[A-Za-z0-9]",lambda x: ("id",x)),
                     ("[+|]",lambda x: ("|",x),("|",100,'left')),
                     ("[\.]",lambda x: (".",""),(".",200,'left')),
-                    ("[*]",lambda x: (x,x),("*",300,'left')),
+                    ("[*]",lambda x: (x,x), ("*",300,'left')),
                     ("\(|\)",lambda x: (x,x)) ]
         Yappy.__init__(self,tokenize,grammar,table,no_table,tabletype)
 
@@ -349,6 +350,10 @@ class RegExp(Yappy):
      def test(self):
          st  = ["(a+b)*.a.a.b*",
                 "a+a.b+a.b.(a+a)*",
+                "a+a.b+a.(a+a)**",
+                 "a+a.b.c",
+                "a+a.b.(c+b)",
+                "a+a.b.(c+b)*",
                  "a+a.b*.(a+b)"]
          for i in st:
              print "Input: %s" %i
@@ -367,7 +372,7 @@ class RegExp1(RegExp):
         tokenize =[
                     ("[A-Za-z0-9]",lambda x: ("id",x)),
                     ("[+|]",lambda x: ("+",x),("+",100,'left')),
-                    ("[*]",lambda x: (x,x),("*",300,'left')),
+                    ("[*]",lambda x: (x,x)),
                     ("\(|\)",lambda x: (x,x)) ]
         Yappy.__init__(self,tokenize,grammar,table,no_table,tabletype,noconflicts,expect)
         
@@ -379,6 +384,10 @@ class RegExp1(RegExp):
          st  = ["(a+b)*aab*",
                 "(a+ab)*a*",
                 "(a+a)a+ab",
+                "a+ab+(a(a+a)*)*",
+                "a+ab+a(a+a)**",
+                "(a+a)**ab(a+b)**",
+                "aa+bb**",
                 "(a+ab)(a+ab)(ac+a)",
                 "a+abc+ad",
                 "abc+b+ad",
@@ -392,7 +401,9 @@ class RegExp1(RegExp):
                 "a+c+ab(a+b)",
                 "a+c+(a+b)ab",
                 "a+b*",
-                "aa+b*"
+                "aa+b*",
+                "aab*ab+a*+aa",
+                "aab*ab**+(a+aa)**"
                 ]
          for i in st:
              print "Input: %s" %i
@@ -412,7 +423,7 @@ class RegExp2(RegExp1):
                     ("@empty_set",lambda x: ("id",x)),
                     ("[A-Za-z0-9]",lambda x: ("id",x)),
                     ("[+|]",lambda x: ("+",x),("+",100,'left')),
-                    ("[*]",lambda x: (x,x),("*",300,'left')),
+                    ("[*]",lambda x: (x,x)),
                     ("\(|\)",lambda x: (x,x)) ]
 
          Yappy.__init__(self,tokenize,grammar,table,no_table)
